"use strict";

/*
 * version: 0.1.1
 * updated:
 *  当 DOM Inspector 未安装时，如果查找的是 WEB 中的元素，将尝试：
 *      当安装装了 Firebug，将使用 Firebug 来定位元素的 DOM 位置；
 *      否则尝试通过 Firefox 自带的（仅支持 Firefox 17+）Inspector 来定位元素。
 */

const {classes: Cc, interfaces: Ci, utils: Cu} = Components;

Cu.import("resource://gre/modules/PopupNotifications.jsm");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/AddonManager.jsm");

const stringBundle = Services.strings.createBundle("chrome://InspectElement/locale/global.properties?" + Math.random()); // Randomize URI to work around bug 719376

var InspectElement = {
    disabled: true,
    ww: Services.ww,       // nsIWindowWatcher
    wm: Services.wm,       // nsIWindowMediator

    get isWinNT() {
        var os = Services.appinfo.OS;
        return os == "WINNT" ? true : false;
    },

    handleEvent: function(e) {
        // Shift + 右键 响应
        if (!e.shiftKey || e.button != 2) return;
        try {
            e.stopPropagation();
            e.preventDefault();
        } catch (ex) {}
        if (e.type != "click") return;
        let elem = e.originalTarget;
        let shadowElem = e.target;
        //if (this.disabled) return this.error();
        let win = e.currentTarget;
        if (!this.dependenciesChecked) {
            this.checkDependencies(this.inspect.bind(this, win, elem, shadowElem));
        } else {
            this.inspect(win, elem, shadowElem);
        }
    },
    inspect: function(win, elem, shadowElem) {
        if (!this.disabled) {
            win.openDialog("chrome://inspector/content/", "_blank",
                           "chrome, all, dialog=no", elem);
        } else {
            try {
                if (win.Firebug) {
                    let Firebug = win.Firebug;
                    (function (elem, Firebug) {
                        Firebug.browserOverlay.startFirebug(function (Firebug) {
                            Firebug.Inspector.inspectFromContextMenu(elem);
                        });
                    })(shadowElem, Firebug);
                } else {
                    (function (win, elem, shadowElem) {
                        /*
                         * 有这么变的吗，四个版本，变了三次地址！！！
                         */
                        let devtools = {};
                        let version = Services.appinfo.version.split(".")[0];
                        let DEVTOOLS_URI;
                        if (version >= 53) { // hehe
                            // TODO
                        } else if (version >= 50) {
                            return this.inspectNode50(win, elem, shadowElem);
                        } else if (version >= 24) {
                            DEVTOOLS_URI = "resource://gre/modules/devtools/Loader.jsm";
                            ({devtools} = Cu.import(DEVTOOLS_URI, {}));
                        } else if (version >= 23) {
                            DEVTOOLS_URI = "resource:///modules/devtools/gDevTools.jsm";
                            ({devtools} = Cu.import(DEVTOOLS_URI, {}));
                        } else if (version >= 20) {
                            DEVTOOLS_URI = "resource:///modules/devtools/Target.jsm";
                            devtools = Cu.import(DEVTOOLS_URI, {});
                        } else {
                            return (function (elem, InspectorUI) {
                                if (InspectorUI.isTreePanelOpen) {
                                    InspectorUI.inspectNode(elem);
                                    InspectorUI.stopInspecting();
                                } else {
                                    InspectorUI.openInspectorUI(elem);
                                }
                            })(shadowElem, win.InspectorUI);
                        }
                        let gBrowser = win.gBrowser, gDevTools = win.gDevTools;
                        let tt = devtools.TargetFactory.forTab(gBrowser.selectedTab);
                        return gDevTools.showToolbox(tt, "inspector").then((function (elem) {
                            return function(toolbox) {
                                let inspector = toolbox.getCurrentPanel();
                                inspector.selection.setNode(elem, "Extension-Element-Inspector");
                            }
                        })(shadowElem));
                    }).call(this, win, elem, shadowElem);
                }
            } catch (ex) {
                this.error();
            }
        }
        this.closePopup(elem, win);
    },
    inspectNode50(win, elem, shadowElem) {
        const { devtools } = Cu.import("resource://devtools/shared/Loader.jsm", {});
        const { gBrowser, gDevTools } = win;
        const target = devtools.TargetFactory.forTab(gBrowser.selectedTab);
        return gDevTools.showToolbox(target, "inspector").then(toolbox => {
            const inspector = toolbox.getCurrentPanel();

            const onNewNode = inspector.selection.once("new-node-front");

            gBrowser.mCurrentBrowser.frameLoader.messageManager.sendAsyncMessage("debug:inspect", {}, {
                node: shadowElem,
            });
            inspector.walker.findInspectingNode().then(nodeFront => {
                inspector.selection.setNodeFront(nodeFront, "Extension-Element-Inspector");
            });

            return onNewNode.then(() => {
                return inspector.once("inspector-updated");
            });
        });
    },
    closePopup: function (elem, win) {
        var parent = elem.parentNode;
        var list = [];
        while (parent != win && parent != null) {
            if (parent.localName == "menupopup" || parent.localName == "popup") {
                list.push(parent);
            }
            parent = parent.parentNode;
        }
        var len = list.length;
        if (!len) return;
        list[len - 1].hidePopup();
    },

    aListener: {
        onOpenWindow: function (aWindow) {
            var win = aWindow.docShell.QueryInterface(
                      Ci.nsIInterfaceRequestor).getInterface(Ci.nsIDOMWindow);
            win.addEventListener("load", function _() {
                this.removeEventListener("load", _, false);
                win.addEventListener("click", InspectElement, true);
                // fix context menu bug in linux
                if (InspectElement.isWinNT) return;
                //win.addEventListener("mousedown", InspectElement, true);
                win.addEventListener("mouseup", InspectElement, false);
                win.addEventListener("contextmenu", InspectElement, true);
            }, false);
        },
        onCloseWindow: function (aWindow) {},
        onWindowTitleChange: function (aWindow, aTitle) {},
    },

    startup: function () {
        this.wm.addListener(this.aListener);
        var cw = this.ww.getWindowEnumerator();
        while (cw.hasMoreElements()) {
            var win = cw.getNext().QueryInterface(Ci.nsIDOMWindow);
            win.addEventListener("click", InspectElement, true);
            // fix context menu bug in linux
            if (this.isWinNT) continue;
            //win.addEventListener("mousedown", InspectElement, true);
            win.addEventListener("mouseup", InspectElement, false);
            win.addEventListener("contextmenu", InspectElement, true);
        }
    },
    checkDependencies: function (done) {
        this.dependenciesChecked = true;
        let that = this;
        AddonManager.getAllAddons(function (addons) {
            for (let i = 0, len = addons.length; i < len; i++) {
                if ((addons[i].id == "inspector@mozilla.org" || addons[i].id == "inspector-dp@mozilla.org") && addons[i].isActive) {
                    that.disabled = false;
                    break;
                }
            }
            done();
        });
    },
    shutdown: function () {
        this.wm.removeListener(this.aListener);
        var cw = this.ww.getWindowEnumerator();
        while (cw.hasMoreElements()) {
            var win = cw.getNext().QueryInterface(Ci.nsIDOMWindow);
            win.removeEventListener("click", InspectElement, true);
            if (this.isWinNT) continue;
            //win.removeEventListener("mousedown", InspectElement, true);
            win.removeEventListener("mouseup", InspectElement, false);
            win.removeEventListener("contextmenu", InspectElement, true);
        }
    },

    observe: function (subject, topic, data) {
        if (topic == "alertclickcallback" && data == "link") {
            var win = this.wm.getMostRecentWindow("navigator:browser");
            var url = "https://addons.mozilla.org/en-US/firefox/addon/dom-inspector-6622/";
            if (win && win.gBrowser) {
                win.gBrowser.loadOneTab(url, null, null, null, false, false);
            } else {
                this.ww.openWindow(win ? win : null, url, win.name, null, null);
            }
        }
    },
    error: function () {
        var et = stringBundle.GetStringFromName("required");
        var as = Cc["@mozilla.org/alerts-service;1"].getService(Ci.nsIAlertsService);
        as.showAlertNotification("chrome://global/skin/icons/Error.png",
                                 "Error:", et, true, "link", this,
                                 "InspectElementError");
        return;
    }
}

// 启用
function startup(data, reason) {
    InspectElement.startup();
}

// 禁用或应用程序退出
function shutdown(data, reason) {
    InspectElement.shutdown();
}

// 安装
function install(data, reason) {
}

// 卸载
function uninstall(data, reason) {
}
